
# encoder.py
# Andrew Davison, ad@coe.psu.ac.th, August 2024

'''
  Convert a command line string into an image made up
  of letter pics loaded from the subdirectory letters/
  or framed/

  Image construction uses the Pillow module.

  The image dimensions are roughly square, and spaces 
  are randomly added to the input string to make it a
  suitable length.

  Usage:
     python encoder.py [-f] "string"
  e.g.
     python encoder.py -f "This is a top secret message"

  The letter images come from  Erik Demaine, Martin Demaine, Donald E. Knuth, and Yushi Uno's 2-piece connected 
  Dissection Font at:
     https://erikdemaine.org/fonts/dissect/?font=2piece
'''

import random, math, sys
from PIL import Image

SAVE_FNM = "encoded.png"


def getDimensions(n):
  # factor n into two integers closest to a square of area n
  best = sys.maxsize
  root = int(math.ceil(math.sqrt(n)))
  a = 1; b = n
  for i in range(1, root+1):
    j = int(round(n/i))
    if i*j >= n:
      diff = abs(i - j)
      if diff < best:
        best = diff
        a = i
        b = j
  return a, b

# -----------------------------------

nArgs = len(sys.argv)
if nArgs < 2 or nArgs > 3:
  print("Usage: python encoder.py [-f] \"string\"")
  sys.exit()

letsDir = "letters" if nArgs == 2 else "framed"
str = sys.argv[1] if nArgs == 2 else sys.argv[2]

img = Image.open(letsDir + "/let_a.png")
letWidth, letHeight = img.size

# simplify text to lowercase and spaces
chs = list(str.lower())
for i in range(len(chs)):
  if not (chs[i].isalnum() or chs[i] == ' '):
    chs[i] = ' '

n = len(chs)
rows, cols = getDimensions(n)
nExtras = rows*cols - n
print("rows:", rows, "; cols:", cols, "; extras:", nExtras)

# add random spaces to fill grid of rows*cols
for i in range(nExtras):
  pos = random.randrange(len(chs))
  chs.insert(pos, ' ')

# build picture grid and print char grid
pic = Image.new("RGB", (cols*letWidth, rows*letHeight))
print("Grid layout:")
for r in range(rows):
  for c in range(cols):
    let = chs[(r*cols)+c]
    print(let, end=' ')
    if let == ' ':
      fnm = "let_Sp.png"
    else:
      fnm = "let_" + let + ".png"
    letIm = Image.open(letsDir + "/" + fnm)
    pic.paste(letIm, (c*letWidth, r*letHeight))
  print()
print()

print("Saving picture to", SAVE_FNM)
pic.save(SAVE_FNM)
pic.close()
with Image.open(SAVE_FNM) as im: 
  im.show()

